@@ -3,9 +3,10 @@ package My::Module::Build;
 use strict;
 use warnings;
 
-use base qw{ Module::Build };
+use Module::Build;
+our @ISA = qw{ Module::Build };
 
-our $VERSION = '0.036';
+our $VERSION = '0.037';
 
 use Carp;
 
@@ -90,7 +91,7 @@ C<--test_files> argument.
 =head1 SUPPORT
 
 Support is by the author. Please file bug reports at
-L<http://rt.cpan.org>, or in electronic mail to the author.
+L<http://rt.cpan.org/>, or in electronic mail to the author.
 
 =head1 AUTHOR
 
@@ -98,7 +99,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009-2016 by Thomas R. Wyant, III
+Copyright (C) 2009-2018 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
